package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"math/rand"
	"strconv"
	"time"
	"xorm.io/xorm"
)

/*
EstimateUserPosition 预估用户位置
total 排名总数
level 预估排名层级
levelRank  用户当前层级排名
rank  用户总排名
previousRow  上一层级
diffValue  距离上一层级差值
*/
func EstimateUserPosition(engine *xorm.Engine, uid int, dbName string) (total, level, levelRank, rank, previousRow int, diffValue string, err error) {
	//1、查找 `user_public_platoon_setting` 基础设置
	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	if userPublicPlatoonSetting == nil {
		return
	}
	//2、查询排名
	rank, total, userAmount, err := CalcUserRank(engine, zhios_order_relate_utils.IntToStr(uid))
	if err != nil {
		return
	}
	if rank == 0 {
		userPublicPlatoonDoubleNetworkSetting, err1 := db.UserPublicPlatoonDoubleNetworkSettingGetOneByParams(engine, map[string]interface{}{
			"key":   "is_open",
			"value": 1,
		})
		if err1 != nil {
			err = err1
			return
		}
		coinAmount, err2 := svc.GetUserCoinAmount(engine.NewSession(), dbName, userPublicPlatoonDoubleNetworkSetting.CoinId, uid)
		if err2 != nil {
			err = err2
			return
		}
		userProfile, err3 := db.UserProfileFindByID(engine, uid)
		if err3 != nil {
			err = err3
			return
		}
		_, err4 := db.UserPublicPlatoonDoubleNetworkUserCoinRecordInsert(engine, &model.UserPublicPlatoonDoubleNetworkUserCoinRecord{
			Uid:          uid,
			LastAmount:   coinAmount,
			Amount:       coinAmount,
			RecommendUid: userProfile.ParentUid,
			CoinId:       userPublicPlatoonDoubleNetworkSetting.CoinId,
			CreateAt:     time.Now().Format("2006-01-02 15:04:05"),
			UpdateAt:     time.Now().Format("2006-01-02 15:04:05"),
		})
		if err4 != nil {
			err = err4
			return
		}
	}
	rows := float64(userPublicPlatoonSetting.SeveralRows)
	times := float64(userPublicPlatoonSetting.SeveralTimes)
	level = makeSearchLevelByPosition(&rank, rows, &times)
	levelPosition1 := getLevelForFirstPosition(level, userPublicPlatoonSetting.SeveralTimes)
	levelRank = rank - levelPosition1 + 1

	//3、计算与前排差距
	previousRow = level - 1
	if previousRow > 0 {
		previousRowPosition1 := getLevelForLastPosition(previousRow, userPublicPlatoonSetting.SeveralTimes)
		previousRowAmount, err1 := GetUserRankAmount(engine, strconv.Itoa(previousRowPosition1))
		if err1 != nil {
			err = err1
			return
		}
		userAmountValue, _ := decimal.NewFromString(userAmount)
		previousRowAmountValue, _ := decimal.NewFromString(previousRowAmount)
		diffValue = previousRowAmountValue.Sub(userAmountValue).String()
	}
	return
}

/*
NowUserPosition 当前用户位置
level 当前排名
levelRank  用户当前层级排名
*/
func NowUserPosition(engine *xorm.Engine, uid int, dbName string) (level, levelRank int, err error) {
	//1、查找 `user_public_platoon_setting` 基础设置
	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	if userPublicPlatoonSetting == nil {
		return
	}
	//2、查询出 `user_public_platoon_relation` 中相关记录
	userPublicPlatoonRelation, err := db.UserPublicPlatoonRelationGetOneByParams(engine, map[string]interface{}{
		"key":   "uid",
		"value": uid,
	})
	if err != nil {
		return
	}
	if userPublicPlatoonRelation == nil {
		err = errors.New("未查询到当前用户公网记录")
		return
	}
	level = userPublicPlatoonRelation.Level
	levelPosition1 := getLevelForFirstPosition(level, userPublicPlatoonSetting.SeveralTimes)
	levelRank = userPublicPlatoonRelation.Position - levelPosition1 + 1
	return
}

func CalcUserRank(engine *xorm.Engine, uid string) (rank, total int, amount string, err error) {
	sql := "SELECT id, uid, amount, @rank := @rank + 1 AS rank FROM `user_public_platoon_double_network_user_coin_record`, (SELECT @rank:=0) r ORDER BY amount DESC;"
	nativeString, _ := db.QueryNativeString(engine, sql)
	total = len(nativeString)
	if total <= 0 {
		err = errors.New("当前无排名数据")
		return
	}
	for _, v := range nativeString {
		if uid == v["uid"] {
			rank = zhios_order_relate_utils.StrToInt(v["rank"])
			amount = v["amount"]
			break
		}
	}
	return
}

func GetUserRankAmount(engine *xorm.Engine, rank string) (amount string, err error) {
	sql := "SELECT id, uid, amount, @rank := @rank + 1 AS rank FROM `user_public_platoon_double_network_user_coin_record`, (SELECT @rank:=0) r ORDER BY amount DESC;"
	nativeString, _ := db.QueryNativeString(engine, sql)
	if len(nativeString) <= 0 {
		err = errors.New("当前无排名数据")
		return
	}
	for _, v := range nativeString {
		if rank == v["rank"] {
			amount = v["amount"]
			break
		}
	}
	return
}

func ResetPublicPlatoonDoubleNetwork(engine *xorm.Engine, dbName string) (err error) {
	fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!dbName!!!!!!!!!!!!!!!!!!!!", dbName)
	//1、查找 `user_public_platoon_double_network_setting` 基础设置
	now := time.Now()
	userPublicPlatoonDoubleNetworkSetting, err := db.UserPublicPlatoonDoubleNetworkSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return err
	}
	if userPublicPlatoonDoubleNetworkSetting.IsOpen != 1 {
		return errors.New("未开启公排双网！")
	}
	if userPublicPlatoonDoubleNetworkSetting.SettlementDate != now.Format("2006-01-02") {
		return errors.New("非重置日期！")
	}

	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()

	session.Begin()
	//2、进行数据清理
	sql1 := "DROP TABLE `user_public_platoon_relation_1`"                                               //删除备份表
	sql2 := "CREATE TABLE `user_public_platoon_relation_1` LIKE `user_public_platoon_relation`"         //复制表结构
	sql3 := "INSERT INTO `user_public_platoon_relation_1` SELECT * FROM `user_public_platoon_relation`" //复制表数据
	sql4 := "TRUNCATE TABLE `user_public_platoon_relation`;"                                            //截断表
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql1)
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err___SQl1", err.Error())
	}
	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql2)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql3)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql4)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//2、先插入创始人
	//2.1 查找 `user_public_platoon_setting` 基础设置
	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	//2.2 插入数据
	_, err = db.UserPublicPlatoonRelationInsertBySession(session, &model.UserPublicPlatoonRelation{
		Uid:                   userPublicPlatoonSetting.OriginatorUid,
		FatherUid:             "",
		Pid:                   0,
		RecommendUid:          0,
		Level:                 1,
		Position:              1,
		UniqueIdentifier:      "0-" + zhios_order_relate_utils.IntToStr(userPublicPlatoonDoubleNetworkSetting.OriginatorUid) + "-1-1",
		ReturnCommissionNum:   0,
		JoinAt:                now,
		WaitForSettlementDate: now.AddDate(0, 0, userPublicPlatoonSetting.SettleDay).Format("2006-01-02"),
		CreateAt:              now,
		UpdateAt:              now,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	err = session.Begin()
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//3、计算排名数据
	sql := "SELECT id, uid,recommend_uid, amount, @rank := @rank + 1 AS rank FROM `user_public_platoon_double_network_user_coin_record`, (SELECT @rank:=0) r ORDER BY amount DESC;"
	nativeString, _ := db.QueryNativeString(engine, sql)
	if len(nativeString) <= 0 {
		return errors.New("当前无排名数据")
	}
	for _, v := range nativeString {
		//3.1 加入公排
		_, err1 := AddPublicPlatoonRelateCommission(engine, []*md.AddPublicPlatoonRelateCommissionReq{
			{
				Uid:          v["uid"],
				RecommendUid: v["recommend_uid"],
			},
		})
		if err1 != nil {
			return err1
		}
		//3.2将`user_public_platoon_double_network_user_coin_record` 中 amount 置0
		var tmpSql = fmt.Sprintf("UPDATE table_name SET amount = 0 WHERE id=%s", v["id"])
		_, err2 := db.ExecuteOriginalSql(engine, tmpSql)
		if err2 != nil {
			return err2
		}
	}

	//4、修改 user_public_platoon_double_network_setting 中的结算时间
	userPublicPlatoonDoubleNetworkSetting.SettlementDate = now.AddDate(0, 1, 0).Format("2006-01-02")
	updateAffected, err := db.UserPublicPlatoonDoubleNetworkSettingUpdate(engine, userPublicPlatoonDoubleNetworkSetting.Id, userPublicPlatoonDoubleNetworkSetting, "settlement_date")
	if err != nil {
		return err
	}
	if updateAffected <= 0 {
		fmt.Println("updateAffected:::::::::", updateAffected)
		return errors.New("更新结算时间失败")
	}
	return
}
